require 'net/ssh'
require 'net/ssh/connection/session'

module Extensions
  module Net
    module SSH
      module Connection
        module Session
          def exec(command, status: nil, &block)
            return super(command, status: status, &block) unless localhost?

            result = exec_local_command(command)
  
            # Return status if given
            if status
              status[:exit_code] = result[:code]
              status[:exit_signal] = result[:signal]
            end
  
            # Process results
            if result[:error].nil?
              if block
                block.call(nil, :stdout, result[:output])
              else
                $stdout.print(result[:output])
              end
            else
              if block
                block.call(nil, :stderr, result[:error])
              else
                $stderr.print(result[:error])
              end
            end
          end
  
          def send_data(data)
            return super(data) unless localhost?

            binding.pry
            raise 'what do do here?'
            # stdin << data # maybe??
          end

          private
  
          def localhost?
            host == 'localhost' && options[:user].nil?
          end
  
          def exec_local_command(command)
            result = {}
            begin
              result[:output] = `#{command}`
            rescue StandardError => e
              result[:error] = e.message
            ensure
              result[:code] = $?.exitstatus
              result[:signal] = $?.termsig
              result
            end
          end
        end
      end
    end
  end
end

Net::SSH::Connection::Session.prepend(Extensions::Net::SSH::Connection::Session)
